#ifndef _LINUX_MODVERSIONS_H
#define _LINUX_MODVERSIONS_H
#include <linux/modsetver.h>
#include <linux/modules/adb.ver>
#include <linux/modules/af_ax25.ver>
#include <linux/modules/af_ipx.ver>
#include <linux/modules/af_netlink.ver>
#include <linux/modules/bsd_comp.ver>
#include <linux/modules/cdrom.ver>
#include <linux/modules/console.ver>
#include <linux/modules/context.ver>
#include <linux/modules/ddp.ver>
#include <linux/modules/fatfs_syms.ver>
#include <linux/modules/fbcmap.ver>
#include <linux/modules/fbcon-cfb8.ver>
#include <linux/modules/fbcon-mfb.ver>
#include <linux/modules/fbcon.ver>
#include <linux/modules/fbmem.ver>
#include <linux/modules/fc_syms.ver>
#include <linux/modules/ftape_syms.ver>
#include <linux/modules/irmod.ver>
#include <linux/modules/ksyms.ver>
#include <linux/modules/lapb_iface.ver>
#include <linux/modules/ll_rw_blk.ver>
#include <linux/modules/lockd_syms.ver>
#include <linux/modules/loop.ver>
#include <linux/modules/md.ver>
#include <linux/modules/misc.ver>
#include <linux/modules/msdosfs_syms.ver>
#include <linux/modules/netsyms.ver>
#include <linux/modules/newport.ver>
#include <linux/modules/nls_base.ver>
#include <linux/modules/nubus_syms.ver>
#include <linux/modules/p8022.ver>
#include <linux/modules/pcisyms.ver>
#include <linux/modules/ppp.ver>
#include <linux/modules/ppp_deflate.ver>
#include <linux/modules/procfs_syms.ver>
#include <linux/modules/psnap.ver>
#include <linux/modules/pty.ver>
#include <linux/modules/scsi_syms.ver>
#include <linux/modules/selection.ver>
#include <linux/modules/signal.ver>
#include <linux/modules/slhc.ver>
#include <linux/modules/sparc_ksyms.ver>
#include <linux/modules/sunrpc_syms.ver>
#include <linux/modules/vfatfs_syms.ver>
#include <linux/modules/wanmain.ver>
#include <linux/modules/zftape_syms.ver>
#include <linux/modules/zorrosyms.ver>
#endif
