( cd usr/src ; rm -rf linux )
( cd usr/src ; ln -sf linux-2.2.20pre2 linux )
( cd usr/src/linux-2.2.20pre2/include ; rm -rf asm )
( cd usr/src/linux-2.2.20pre2/include ; ln -sf asm-sparc asm )
( cd usr/include ; rm -rf linux )
( cd usr/include ; ln -sf /usr/src/linux/include/linux linux )
( cd usr/include ; rm -rf asm )
( cd usr/include ; ln -sf /usr/src/linux/include/asm asm )
( cd usr/include ; rm -rf scsi )
( cd usr/include ; ln -sf /usr/src/linux/include/scsi scsi )
