/*
 *  include/asm-s390/md.h
 *
 *  S390 version
 *
 *  Derived from "include/asm-i386md/.h"
 */
 
#ifndef __ASM_MD_H
#define __ASM_MD_H

/* #define HAVE_ARCH_XORBLOCK */

#define MD_XORBLOCK_ALIGNMENT	sizeof(long)

#endif /* __ASM_MD_H */
